/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
import { TemplateLiteralParseExact, IsTemplateLiteralFinite, TemplateLiteralToUnion } from '../template-literal/index.mjs';
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
import { TTemplateLiteral as IsTemplateLiteralType, TUnionLiteral as IsUnionLiteralType, TUnion as IsUnionType, TLiteral as IsLiteralType, TNumber as IsNumberType, TInteger as IsIntegerType } from '../guard/type.mjs';
// prettier-ignore
function FromTemplateLiteral(T) {
    const E = TemplateLiteralParseExact(T.pattern);
    const F = IsTemplateLiteralFinite(E);
    const S = TemplateLiteralToUnion(T);
    return (F === true
        ? IsUnionLiteralType(S)
            ? S.anyOf.map(S => S.const.toString())
            : []
        : []);
}
// prettier-ignore
function FromUnion(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [...IndexPropertyKeys(L), ...FromUnion(R)]
        : []);
}
// prettier-ignore
function FromLiteral(T) {
    return ([T.toString()]);
}
/** Returns a tuple of PropertyKeys derived from the given TSchema */
// prettier-ignore
export function IndexPropertyKeys(T) {
    return [...new Set((IsTemplateLiteralType(T) ? FromTemplateLiteral(T) :
            IsUnionType(T) ? FromUnion(T.anyOf) :
                IsLiteralType(T) ? FromLiteral(T.const) :
                    IsNumberType(T) ? ['[number]'] :
                        IsIntegerType(T) ? ['[number]'] :
                            []))];
}
