/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
import { Never } from '../never/index.mjs';
import { IntersectEvaluated } from '../intersect/index.mjs';
import { UnionEvaluated } from '../union/index.mjs';
import { CloneType } from '../clone/type.mjs';
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
import { TArray as IsArrayType, TIntersect as IsIntersectType, TObject as IsObjectType, TNever as IsNeverType, TTuple as IsTupleType, TUnion as IsUnionType } from '../guard/type.mjs';
// prettier-ignore
function FromRest(T, K) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [FromKey(L, K), ...FromRest(R, K)]
        : []);
}
// prettier-ignore
function FromIntersectRest(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsNeverType(L)
            ? [...FromIntersectRest(R)]
            : [L, ...FromIntersectRest(R)]
        : []);
}
// prettier-ignore
function FromIntersect(T, K) {
    return (IntersectEvaluated(FromIntersectRest(FromRest(T, K))));
}
// prettier-ignore
function FromUnionRest(T, S = T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsNeverType(L)
            ? []
            : FromUnionRest(R, S)
        : S);
}
// prettier-ignore
function FromUnion(T, K) {
    return (UnionEvaluated(FromUnionRest(FromRest(T, K))));
}
// prettier-ignore
function FromTuple(T, K) {
    return (K in T ? T[K] :
        K === '[number]' ? UnionEvaluated(T) :
            Never());
}
// prettier-ignore
function FromArray(T, K) {
    return (K === '[number]'
        ? T
        : Never());
}
// prettier-ignore
function FromProperty(T, K) {
    return (K in T
        ? T[K]
        : Never());
}
// prettier-ignore
function FromKey(T, K) {
    return (IsIntersectType(T) ? FromIntersect(T.allOf, K) :
        IsUnionType(T) ? FromUnion(T.anyOf, K) :
            IsTupleType(T) ? FromTuple(T.items ?? [], K) :
                IsArrayType(T) ? FromArray(T.items, K) :
                    IsObjectType(T) ? FromProperty(T.properties, K) :
                        Never());
}
// prettier-ignore
function FromKeys(T, K) {
    const [L, ...R] = K;
    return (K.length > 0
        ? [FromKey(T, L), ...FromKeys(T, R)]
        : []);
}
// prettier-ignore
export function IndexResult(T, K, options) {
    return (CloneType(UnionEvaluated(FromKeys(T, K)), options));
}
