import type { TSchema } from '../schema/index.mjs';
import { type ZeroString, type UnionToTuple, Increment } from '../helpers/index.mjs';
import type { TRecursive } from '../recursive/index.mjs';
import type { TIntersect } from '../intersect/index.mjs';
import type { TUnion } from '../union/index.mjs';
import type { TTuple } from '../tuple/index.mjs';
import type { TArray } from '../array/index.mjs';
import type { TObject, TProperties } from '../object/index.mjs';
import { OperatorUnionMany, OperatorIntersectMany } from '../operators/index.mjs';
type FromRest<T extends TSchema[]> = (T extends [infer L extends TSchema, ...infer R extends TSchema[]] ? [KeyOfPropertyKeys<L>, ...FromRest<R>] : []);
declare function FromRest<T extends TSchema[]>(T: [...T]): FromRest<T>;
type FromIntersect<T extends TSchema[], C extends PropertyKey[][] = FromRest<T>> = (OperatorUnionMany<C>);
declare function FromIntersect<T extends TSchema[]>(T: [...T], C?: FromRest<T>): FromIntersect<T>;
type FromUnion<T extends TSchema[], C extends PropertyKey[][] = FromRest<T>> = (OperatorIntersectMany<C>);
declare function FromUnion<T extends TSchema[]>(T: [...T]): FromUnion<T>;
type FromTuple<T extends TSchema[], I extends string = ZeroString> = T extends [infer _ extends TSchema, ...infer R extends TSchema[]] ? [I, ...FromTuple<R, Increment<I>>] : [];
declare function FromTuple<T extends TSchema[]>(T: [...T], I?: string): FromTuple<T>;
type FromArray<_ extends TSchema> = ([
    '[number]'
]);
declare function FromArray<_ extends TSchema>(_: _): FromArray<_>;
type FromProperties<T extends TProperties> = (UnionToTuple<keyof T>);
declare function FromProperties<T extends TProperties>(T: T): FromProperties<T>;
export type KeyOfPropertyKeys<T extends TSchema> = (T extends TRecursive<infer S> ? KeyOfPropertyKeys<S> : T extends TIntersect<infer S> ? FromIntersect<S> : T extends TUnion<infer S> ? FromUnion<S> : T extends TTuple<infer S> ? FromTuple<S> : T extends TArray<infer S> ? FromArray<S> : T extends TObject<infer S> ? FromProperties<S> : [
]);
/** Returns a tuple of PropertyKeys derived from the given TSchema. */
export declare function KeyOfPropertyKeys<T extends TSchema>(T: T): KeyOfPropertyKeys<T>;
/** Returns a regular expression pattern derived from the given TSchema */
export declare function KeyOfPattern(schema: TSchema): string;
export {};
