/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
import { Intersect } from '../intersect/index.mjs';
import { Union } from '../union/index.mjs';
import { Optional } from '../optional/index.mjs';
import { Discard } from '../discard/index.mjs';
import { OptionalKind } from '../symbols/index.mjs';
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
import { TOptional as IsOptionalType } from '../guard/type.mjs';
// prettier-ignore
function IsOptionalFromIntersect(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsOptionalType(L)
            ? IsOptionalFromIntersect(R)
            : false
        : true);
}
// prettier-ignore
function IsOptionalFromUnion(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? IsOptionalType(L)
            ? true
            : IsOptionalFromUnion(R)
        : false);
}
// prettier-ignore
function RemoveOptionalFromRest(T) {
    return (T.map(T => RemoveOptionalFromSchema(T)));
}
// prettier-ignore
function RemoveOptionalFromSchema(T) {
    return (Discard(T, [OptionalKind]));
}
// prettier-ignore
export function OptionalFromIntersect(T) {
    const D = RemoveOptionalFromRest(T);
    return (IsOptionalFromIntersect(T)
        ? Optional(Intersect(D))
        : Intersect(T));
}
// prettier-ignore
export function OptionalFromUnion(T) {
    const D = RemoveOptionalFromRest(T);
    return (IsOptionalFromUnion(T)
        ? Optional(Union(D))
        : Union(T));
}
