"use strict";
/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.Awaited = exports.Resolve = void 0;
const index_1 = require("../intersect/index");
const index_2 = require("../union/index");
const type_1 = require("../clone/type");
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
const type_2 = require("../guard/type");
// prettier-ignore
function AwaitedUnwrap(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [Resolve(L), ...AwaitedUnwrap(R)]
        : []);
}
// prettier-ignore
function AwaitedIntersectRest(T) {
    return (0, index_1.Intersect)(AwaitedUnwrap(T));
}
// prettier-ignore
function AwaitedUnionRest(T) {
    return (0, index_2.Union)(AwaitedUnwrap(T));
}
// prettier-ignore
function AwaitedPromise(T) {
    return Resolve(T);
}
// prettier-ignore
function Resolve(T) {
    return ((0, type_2.TIntersect)(T) ? AwaitedIntersectRest(T.allOf) :
        (0, type_2.TUnion)(T) ? AwaitedUnionRest(T.anyOf) :
            (0, type_2.TPromise)(T) ? AwaitedPromise(T.item) :
                T);
}
exports.Resolve = Resolve;
/** `[JavaScript]` Constructs a type by recursively unwrapping Promise types */
function Awaited(T, options = {}) {
    return (0, type_1.CloneType)(Resolve(T), options);
}
exports.Awaited = Awaited;
