"use strict";
/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.Deref = exports.DerefResolve = void 0;
const type_1 = require("../clone/type");
const index_1 = require("../discard/index");
const value_1 = require("../guard/value");
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
const type_2 = require("../guard/type");
function FromRest(schema, references) {
    return schema.map((schema) => Deref(schema, references));
}
// prettier-ignore
function FromProperties(properties, references) {
    return globalThis.Object.getOwnPropertyNames(properties).reduce((acc, key) => {
        return { ...acc, [key]: Deref(properties[key], references) };
    }, {});
}
// prettier-ignore
function FromConstructor(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.parameters = FromRest(clone.parameters, references);
    clone.returns = Deref(clone.returns, references);
    return clone;
}
// prettier-ignore
function FromFunction(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.parameters = FromRest(clone.parameters, references);
    clone.returns = Deref(clone.returns, references);
    return clone;
}
// prettier-ignore
function FromIntersect(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.allOf = FromRest(clone.allOf, references);
    return clone;
}
// prettier-ignore
function FromUnion(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.anyOf = FromRest(clone.anyOf, references);
    return clone;
}
// prettier-ignore
function FromTuple(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    if ((0, value_1.IsUndefined)(clone.items))
        return clone;
    clone.items = FromRest(clone.items, references);
    return clone;
}
// prettier-ignore
function FromArray(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
// prettier-ignore
function FromObject(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.properties = FromProperties(clone.properties, references);
    return clone;
}
// prettier-ignore
function FromPromise(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.item = Deref(clone.item, references);
    return clone;
}
// prettier-ignore
function FromAsyncIterator(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
// prettier-ignore
function FromIterator(schema, references) {
    const clone = (0, type_1.CloneType)(schema);
    clone.items = Deref(clone.items, references);
    return clone;
}
// prettier-ignore
function FromRef(schema, references) {
    const target = references.find(remote => remote.$id === schema.$ref);
    if (target === undefined)
        throw Error(`Unable to dereference schema with $id ${schema.$ref}`);
    const discard = (0, index_1.Discard)(target, ['$id']);
    return Deref(discard, references);
}
// prettier-ignore
function DerefResolve(schema, references) {
    return ((0, type_2.TConstructor)(schema) ? FromConstructor(schema, references) :
        (0, type_2.TFunction)(schema) ? FromFunction(schema, references) :
            (0, type_2.TIntersect)(schema) ? FromIntersect(schema, references) :
                (0, type_2.TUnion)(schema) ? FromUnion(schema, references) :
                    (0, type_2.TTuple)(schema) ? FromTuple(schema, references) :
                        (0, type_2.TArray)(schema) ? FromArray(schema, references) :
                            (0, type_2.TObject)(schema) ? FromObject(schema, references) :
                                (0, type_2.TPromise)(schema) ? FromPromise(schema, references) :
                                    (0, type_2.TAsyncIterator)(schema) ? FromAsyncIterator(schema, references) :
                                        (0, type_2.TIterator)(schema) ? FromIterator(schema, references) :
                                            (0, type_2.TRef)(schema) ? FromRef(schema, references) :
                                                schema);
}
exports.DerefResolve = DerefResolve;
/** `[Json]` Creates a dereferenced type */
function Deref(schema, references) {
    return DerefResolve(schema, references);
}
exports.Deref = Deref;
