import { Kind, Hint, TransformKind } from '../symbols/index';
import type { TransformOptions } from '../transform/index';
import type { TTemplateLiteral, TTemplateLiteralKind } from '../template-literal/index';
import type { TArray } from '../array/index';
import type { TBoolean } from '../boolean/index';
import type { TRecord } from '../record/index';
import type { TString } from '../string/index';
import type { TUnion } from '../union/index';
import type { TAny } from '../any/index';
import type { TAsyncIterator } from '../async-iterator/index';
import type { TBigInt } from '../bigint/index';
import type { TConstructor } from '../constructor/index';
import type { TFunction } from '../function/index';
import type { TInteger } from '../integer/index';
import type { TIntersect } from '../intersect/index';
import type { TIterator } from '../iterator/index';
import type { TLiteral, TLiteralValue } from '../literal/index';
import type { TMappedKey, TMappedResult } from '../mapped/index';
import type { TNever } from '../never/index';
import type { TNot } from '../not/index';
import type { TNull } from '../null/index';
import type { TNumber } from '../number/index';
import type { TObject, TProperties } from '../object/index';
import type { TOptional } from '../optional/index';
import type { TPromise } from '../promise/index';
import type { TReadonly } from '../readonly/index';
import type { TRef } from '../ref/index';
import type { TSchema } from '../schema/index';
import type { TSymbol } from '../symbol/index';
import type { TTuple } from '../tuple/index';
import type { TUint8Array } from '../uint8array/index';
import type { TUndefined } from '../undefined/index';
import type { TUnknown } from '../unknown/index';
import type { TUnsafe } from '../unsafe/index';
import type { TVoid } from '../void/index';
import type { TDate } from '../date/index';
import type { TThis } from '../recursive/index';
export declare class TypeGuardUnknownTypeError extends Error {
}
/** Returns true if this value has a Readonly symbol */
export declare function TReadonly<T extends TSchema>(value: T): value is TReadonly<T>;
/** Returns true if this value has a Optional symbol */
export declare function TOptional<T extends TSchema>(value: T): value is TOptional<T>;
/** Returns true if the given value is TAny */
export declare function TAny(value: unknown): value is TAny;
/** Returns true if the given value is TArray */
export declare function TArray(value: unknown): value is TArray;
/** Returns true if the given value is TAsyncIterator */
export declare function TAsyncIterator(value: unknown): value is TAsyncIterator;
/** Returns true if the given value is TBigInt */
export declare function TBigInt(value: unknown): value is TBigInt;
/** Returns true if the given value is TBoolean */
export declare function TBoolean(value: unknown): value is TBoolean;
/** Returns true if the given value is TConstructor */
export declare function TConstructor(value: unknown): value is TConstructor;
/** Returns true if the given value is TDate */
export declare function TDate(value: unknown): value is TDate;
/** Returns true if the given value is TFunction */
export declare function TFunction(value: unknown): value is TFunction;
/** Returns true if the given value is TInteger */
export declare function TInteger(value: unknown): value is TInteger;
/** Returns true if the given schema is TProperties */
export declare function TProperties(value: unknown): value is TProperties;
/** Returns true if the given value is TIntersect */
export declare function TIntersect(value: unknown): value is TIntersect;
/** Returns true if the given value is TIterator */
export declare function TIterator(value: unknown): value is TIterator;
/** Returns true if the given value is a TKind with the given name. */
export declare function TKindOf<T extends string>(value: unknown, kind: T): value is Record<PropertyKey, unknown> & {
    [Kind]: T;
};
/** Returns true if the given value is TLiteral<string> */
export declare function TLiteralString(value: unknown): value is TLiteral<string>;
/** Returns true if the given value is TLiteral<number> */
export declare function TLiteralNumber(value: unknown): value is TLiteral<number>;
/** Returns true if the given value is TLiteral<boolean> */
export declare function TLiteralBoolean(value: unknown): value is TLiteral<boolean>;
/** Returns true if the given value is TLiteral */
export declare function TLiteral(value: unknown): value is TLiteral;
/** Returns true if the given value is a TLiteralValue */
export declare function TLiteralValue(value: unknown): value is TLiteralValue;
/** Returns true if the given value is a TMappedKey */
export declare function TMappedKey(value: unknown): value is TMappedKey;
/** Returns true if the given value is TMappedResult */
export declare function TMappedResult(value: unknown): value is TMappedResult;
/** Returns true if the given value is TNever */
export declare function TNever(value: unknown): value is TNever;
/** Returns true if the given value is TNot */
export declare function TNot(value: unknown): value is TNot;
/** Returns true if the given value is TNull */
export declare function TNull(value: unknown): value is TNull;
/** Returns true if the given value is TNumber */
export declare function TNumber(value: unknown): value is TNumber;
/** Returns true if the given value is TObject */
export declare function TObject(value: unknown): value is TObject;
/** Returns true if the given value is TPromise */
export declare function TPromise(value: unknown): value is TPromise;
/** Returns true if the given value is TRecord */
export declare function TRecord(value: unknown): value is TRecord;
/** Returns true if this value is TRecursive */
export declare function TRecursive(value: unknown): value is {
    [Hint]: 'Recursive';
};
/** Returns true if the given value is TRef */
export declare function TRef(value: unknown): value is TRef;
/** Returns true if the given value is TString */
export declare function TString(value: unknown): value is TString;
/** Returns true if the given value is TSymbol */
export declare function TSymbol(value: unknown): value is TSymbol;
/** Returns true if the given value is TTemplateLiteral */
export declare function TTemplateLiteral(value: unknown): value is TTemplateLiteral<TTemplateLiteralKind[]>;
/** Returns true if the given value is TThis */
export declare function TThis(value: unknown): value is TThis;
/** Returns true of this value is TTransform */
export declare function TTransform(value: unknown): value is {
    [TransformKind]: TransformOptions;
};
/** Returns true if the given value is TTuple */
export declare function TTuple(value: unknown): value is TTuple;
/** Returns true if the given value is TUndefined */
export declare function TUndefined(value: unknown): value is TUndefined;
/** Returns true if the given value is TUnion<Literal<string | number>[]> */
export declare function TUnionLiteral(value: unknown): value is TUnion<TLiteral[]>;
/** Returns true if the given value is TUnion */
export declare function TUnion(value: unknown): value is TUnion;
/** Returns true if the given value is TUint8Array */
export declare function TUint8Array(value: unknown): value is TUint8Array;
/** Returns true if the given value is TUnknown */
export declare function TUnknown(value: unknown): value is TUnknown;
/** Returns true if the given value is a raw TUnsafe */
export declare function TUnsafe(value: unknown): value is TUnsafe<unknown>;
/** Returns true if the given value is TVoid */
export declare function TVoid(value: unknown): value is TVoid;
/** Returns true if the given value is TKind */
export declare function TKind(value: unknown): value is Record<PropertyKey, unknown> & {
    [Kind]: string;
};
/** Returns true if the given value is TSchema */
export declare function TSchema(value: unknown): value is TSchema;
