"use strict";
/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyOfPattern = exports.KeyOfPropertyKeys = void 0;
const index_1 = require("../helpers/index");
const index_2 = require("../operators/index");
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
const type_1 = require("../guard/type");
// prettier-ignore
function FromRest(T) {
    const [L, ...R] = T;
    return (T.length > 0
        ? [KeyOfPropertyKeys(L), ...FromRest(R)]
        : []);
}
// prettier-ignore
function FromIntersect(T, C = FromRest(T)) {
    return (0, index_2.OperatorUnionMany)(C);
}
// prettier-ignore
function FromUnion(T) {
    const C = FromRest(T);
    return (0, index_2.OperatorIntersectMany)(C);
}
// prettier-ignore
function FromTuple(T, I = '0') {
    const [_, ...R] = T;
    return (T.length > 0
        ? [I, ...FromTuple(R, (0, index_1.Increment)(I))]
        : []);
}
// prettier-ignore
function FromArray(_) {
    return (['[number]']);
}
// prettier-ignore
function FromProperties(T) {
    return (globalThis.Object.getOwnPropertyNames(T));
}
// ------------------------------------------------------------------
// FromPatternProperties
// ------------------------------------------------------------------
// prettier-ignore
function FromPatternProperties(patternProperties) {
    if (!includePatternProperties)
        return [];
    const patternPropertyKeys = globalThis.Object.getOwnPropertyNames(patternProperties);
    return patternPropertyKeys.map(key => {
        return (key[0] === '^' && key[key.length - 1] === '$')
            ? key.slice(1, key.length - 1)
            : key;
    });
}
/** Returns a tuple of PropertyKeys derived from the given TSchema. */
// prettier-ignore
function KeyOfPropertyKeys(T) {
    return ((0, type_1.TIntersect)(T) ? FromIntersect(T.allOf) :
        (0, type_1.TUnion)(T) ? FromUnion(T.anyOf) :
            (0, type_1.TTuple)(T) ? FromTuple(T.items ?? []) :
                (0, type_1.TArray)(T) ? FromArray(T.items) :
                    (0, type_1.TObject)(T) ? FromProperties(T.properties) :
                        (0, type_1.TRecord)(T) ? FromPatternProperties(T.patternProperties) :
                            []);
}
exports.KeyOfPropertyKeys = KeyOfPropertyKeys;
// ----------------------------------------------------------------
// KeyOfPattern
// ----------------------------------------------------------------
let includePatternProperties = false;
/** Returns a regular expression pattern derived from the given TSchema */
function KeyOfPattern(schema) {
    includePatternProperties = true;
    const keys = KeyOfPropertyKeys(schema);
    includePatternProperties = false;
    const pattern = keys.map((key) => `(${key})`);
    return `^(${pattern.join('|')})$`;
}
exports.KeyOfPattern = KeyOfPattern;
