"use strict";
/*--------------------------------------------------------------------------

@sinclair/typebox/type

The MIT License (MIT)

Copyright (c) 2017-2023 Haydn Paterson (sinclair) <haydn.developer@gmail.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

---------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.Record = void 0;
const index_1 = require("../object/index");
const index_2 = require("../never/index");
const index_3 = require("../union/index");
const index_4 = require("../template-literal/index");
const index_5 = require("../patterns/index");
const index_6 = require("../indexed/index");
const index_7 = require("../symbols/index");
const type_1 = require("../clone/type");
const value_1 = require("../guard/value");
// ------------------------------------------------------------------
// TypeGuard
// ------------------------------------------------------------------
// prettier-ignore
const type_2 = require("../guard/type");
// ------------------------------------------------------------------
// FromPattern
// ------------------------------------------------------------------
// prettier-ignore
function FromPattern(pattern, T) {
    return { [index_7.Kind]: 'Record', type: 'object', patternProperties: { [pattern]: (0, type_1.CloneType)(T) } };
}
// ------------------------------------------------------------------
// FromKeys
// ------------------------------------------------------------------
// prettier-ignore
function FromKeys(keys, T) {
    const properties = keys.reduce((acc, key) => ({ ...acc, [key]: (0, type_1.CloneType)(T) }), {});
    return (0, index_1.Object)(properties, { [index_7.Hint]: 'Record' });
}
// prettier-ignore
function FromTemplateLiteralKey(K, T) {
    const expression = (0, index_4.TemplateLiteralParseExact)(K.pattern);
    return ((0, index_4.IsTemplateLiteralFinite)(expression)
        ? FromKeys((0, index_6.IndexPropertyKeys)(K), T)
        : FromPattern(K.pattern, T));
}
// prettier-ignore
function FromUnionKey(K, T) {
    return FromKeys((0, index_6.IndexPropertyKeys)((0, index_3.Union)(K)), T);
}
// prettier-ignore
function FromLiteralKey(K, T) {
    return FromKeys([K.toString()], T);
}
// prettier-ignore
function FromStringKey(K, T) {
    const pattern = (0, value_1.IsUndefined)(K.pattern) ? index_5.PatternStringExact : K.pattern;
    return FromPattern(pattern, T);
}
// prettier-ignore
function FromIntegerKey(_, T) {
    return FromPattern(index_5.PatternNumberExact, T);
}
// prettier-ignore
function FromNumberKey(_, T) {
    return FromPattern(index_5.PatternNumberExact, T);
}
// prettier-ignore
function RecordResolve(K, T) {
    return ((0, type_2.TUnion)(K) ? FromUnionKey(K.anyOf, T) :
        (0, type_2.TTemplateLiteral)(K) ? FromTemplateLiteralKey(K, T) :
            (0, type_2.TLiteral)(K) ? FromLiteralKey(K.const, T) :
                (0, type_2.TString)(K) ? FromStringKey(K, T) :
                    (0, type_2.TInteger)(K) ? FromIntegerKey(K, T) :
                        (0, type_2.TNumber)(K) ? FromNumberKey(K, T) :
                            (0, index_2.Never)());
}
// prettier-ignore
/** `[Json]` Creates a Record type */
function Record(K, T, options = {}) {
    return (0, type_1.CloneType)(RecordResolve(K, T), options);
}
exports.Record = Record;
